/*
** main.h
*/

#ifndef __MAIN_H__
# define __MAIN_H__

#define PROGRAM_VERSION		"0.1.0"

#define NVRAM_DEVICE		"/dev/nvram"

extern char	**environ;

typedef struct	bios_s
{
  char		*name;
  char		*desc;
  unsigned	len_min;
  unsigned	len_max;
  int		char_min;
  int		char_max;
  void		(*pwd_check)
       (int fd,
	struct bios_s *b,
       	char *(*get_next_pwd)(struct bios_s *b, char *pwd),
       	void (*notify_valid_pwd)(const char *pwd));
  void		(*pwd_set_onoff)
       (int fd, struct bios_s *b, int onoff);
}		bios_t;

typedef enum	method_e
{
  M_BRUTE,
  M_STDIN,
}		method_t;

extern bios_t	bioses[];

/* main.c */
void		usage(const char *s);
void		version(const char *s);
void		list(void);
int		main(int argc, char **argv);
/* utils.c */
void		nvram_init(int fd);
void		nvram_corrcks(int fd);
void		read_nbytes_at(int fd, void *addr, int off, int len);
void		write_nbytes_at(int fd, void *addr, int off, int len);
int		pwd_inc(bios_t *b, char *p);
unsigned	generic_16bits_digest(bios_t *b, const unsigned char *pwd);
/* pwd.c */
void		pwd_find(int fd, bios_t *b, method_t method);
char		*get_next_pwd_brute(bios_t *b, char *pwd);
char		*get_next_pwd_stdin(bios_t *b, char *pwd);
void		notify_valid_pwd_print(const char *pwd);
void		pwd_test(int fd, bios_t *b, char *pwd);
char		*get_next_pwd_justone(bios_t *b, char *pwd);
void		notify_valid_pwd_setflag(const char *pwd);
void		pwd_set(int fd, bios_t *b, int state);
/* bios-*.c */
void		awdmod_pwd_check(
	int	fd,
	bios_t	*b,
       	char	*(*get_next_pwd)(bios_t *b, char *pwd),
       	void	(*notify_valid_pwd)(const char *pwd));
void		awdmod_pwd_set_onoff(int fd, bios_t *b, int onoff);
void		awdmedal_pwd_check(
	int	fd,
	bios_t	*b,
       	char	*(*get_next_pwd)(bios_t *b, char *pwd),
       	void	(*notify_valid_pwd)(const char *pwd));

#endif /* !__MAIN_H__ */
