/*
** pwd.c
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <main.h>

/*
** Find valid passwords
*/
void		pwd_find(int fd, bios_t *b, method_t method)
{
  switch (method)
    {
      case M_BRUTE:
	  printf("Searching valid passwords (by bruteforce)...\n");
	  b->pwd_check(fd, b, get_next_pwd_brute, notify_valid_pwd_print);
	  break;
      case M_STDIN:
	  printf("Searching valid passwords (from stdin)...\n");
	  b->pwd_check(fd, b, get_next_pwd_stdin, notify_valid_pwd_print);
	  break;
    }
}

/*
** Generate all possible pwds (bruteforce method). Return them 1 by 1,
** or return NULL when all pwds have been generated.
*/
char		*get_next_pwd_brute(bios_t *b, char *pwd)
{
    unsigned	i;

    if (!pwd)
      {
	/* get_next_pwd_brute() has never been called,
	   the first pwd is initialized */
	pwd  = calloc(1, b->len_max + 1);
	for (i = 0; i < b->len_min; i++)
	    pwd[i] = b->char_min;
	return pwd;
      }
    else
      {
	/* get_next_pwd_brute() has already been called,
	   the pwd is just "incremented" */
	i = pwd_inc(b, pwd);
	if (i)
	    return NULL; /* overflow in pwd_inc() */
	else
	    return pwd;
      }
}

/*
** Read pwds from stdin. Return them 1 by 1, or return NULL when EOF
** is reached.
*/
char		*get_next_pwd_stdin(bios_t *b, char *pwd)
{
    static char	buf_dicarded[1024];

    if (!pwd)
      {
	/* get_next_pwd_stdin() has never been called,
	   the pwd is allocated */
	pwd  = calloc(1, b->len_max + 1);
      }
    if (fgets(pwd, b->len_max + 1, stdin))
      {
	if (pwd[strlen(pwd) - 1] == '\n')
	    pwd[strlen(pwd) - 1] = 0;
	else
	    /* discard the remaining of the long line */
	    fgets(buf_dicarded, sizeof (buf_dicarded), stdin);
	return pwd;
      }
    else
	return NULL;
}

/*
** Notify the user that a pwd is valid by printing it
*/
void		notify_valid_pwd_print(const char *pwd)
{
    printf("%s\n", pwd);
}

/*
** Test the validity of just one pwd
*/
static char		*justone_pwd;
static int		justone_isvalid;
void		pwd_test(int fd, bios_t *b, char *pwd)
{
    justone_pwd = pwd;
    justone_isvalid = 0;
    b->pwd_check(fd, b, get_next_pwd_justone, notify_valid_pwd_setflag);
    printf("'%s' is%s a valid password\n", justone_pwd,
	   justone_isvalid ? "" : " not");
}

/*
** Return just one pwd and NULL the next time the function is called
*/
char		*get_next_pwd_justone(bios_t *b, char *pwd)
{
    if (!pwd)
      {
	/* get_next_pwd_justone() has never been called,
	   the pwd is returned */
	return justone_pwd;
      }
    else
      {
	/* get_next_pwd_justone() has already been called,
	   return NULL */
	return NULL;
      }
    (void)b; /* avoid the "unused parameter" warning */
}

/*
** Notify the user that the pwd is valid by setting a flag
*/
void		notify_valid_pwd_setflag(const char *pwd)
{
    justone_isvalid = 1;
    (void)pwd; /* avoid the "unused parameter" warning */
}

/*
** Set a pwd state (on/off)
*/
void		pwd_set(int fd, bios_t *b, int state)
{
    if (b->pwd_set_onoff)
      {
	b->pwd_set_onoff(fd, b, state);
	printf("Bios supervisor password %sabled\n", state ? "en" : "dis");
      }
    else
	fprintf(stderr, "Enabling or disabling the bios password for the "
		"bios '%s' is not supported:\nthis program does not know "
		"how to do it :-)\n", b->name);
}
