/*
** utils.c
*/

#include <sys/ioctl.h>
#include <linux/nvram.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <main.h>

/*
** Init nvram
*/
void		nvram_init(int fd)
{
  if (-1 == ioctl(fd, NVRAM_INIT))
    perror("ioctl(NVRAM_INIT)"), exit(EXIT_FAILURE);
  printf("Nvram content initialized\n");
}

/*
** Correct nvram checksum
*/
void		nvram_corrcks(int fd)
{
  if (-1 == ioctl(fd, NVRAM_SETCKS))
    perror("ioctl(NVRAM_SETCKS)"), exit(EXIT_FAILURE);
  printf("Nvram checksum corrected\n");
}

/*
** Read <len> bytes from <fd> (at offset <off>) to <addr>
*/
void		read_nbytes_at(int fd, void *addr, int off, int len)
{
  int		ret;

  if (off != lseek(fd, off, SEEK_SET))
    perror("lseek"), exit(EXIT_FAILURE);
  if (len != (ret = read(fd, addr, len)))
    fprintf(stderr, "read() returned %i instead of %i: %s\n",
	    ret, len, strerror(errno)), exit(EXIT_FAILURE);
}

/*
** Write <len> bytes from <addr> to <fd> (at offset <off>)
*/
void		write_nbytes_at(int fd, void *addr, int off, int len)
{
  int		ret;

  if (off != lseek(fd, off, SEEK_SET))
    perror("lseek"), exit(EXIT_FAILURE);
  if (len != (ret = write(fd, addr, len)))
    fprintf(stderr, "write() returned %i instead of %i: %s\n",
	    ret, len, strerror(errno)), exit(EXIT_FAILURE);
}

/*
** "Increment" a pwd
**
** Return value
**  0		Ok
**  1		On overflow
*/
int		pwd_inc(bios_t *b, char *p)
{
  long		i;

  i = strlen(p) - 1;
  while (i >= 0 && ++p[i] > b->char_max)
    {
      p[i] = b->char_min;
      i--;
    }
  if (i < 0)
    {
      if (strlen(p) < b->len_max)
        {
          for (i = 0; p[i]; i++)
            p[i] = b->char_min;
          p[i] = b->char_min;
          p[i + 1] = 0;
          return 0;
        }
      else
        return 1;
    }
  else
    return 0;
}

/*
** Generic function that compute the digest value associated to a pwd.
** It seems that many bioses use an equivalent function, that's why it
** is present here, in utils.c
*/
unsigned	generic_16bits_digest(bios_t *b, const unsigned char *pwd)
{
    unsigned long	val32;
    unsigned		i;

    for (val32 = i = 0; pwd[i] && i < b->len_max; i++)
	val32 = (val32 << 2) + pwd[i];
    return (val32 & 0xffff) + ((val32 >> 16) & 0xffff); /* 32 -> 16 bits */
}
